@@ -147,7 +147,15 @@ class BaseLLMProvider(ABC):
             Provider-specific message format
         """
         formatted = []
-        for msg in messages:
+        for i, msg in enumerate(messages):
+            # Validate message is dict-like
+            if not isinstance(msg, dict):
+                raise ValueError(f"Message at index {i} must be a dict, got {type(msg).__name__}")
+            
+            # Validate required fields
+            if 'role' not in msg:
+                raise ValueError(f"Message at index {i} missing required field 'role'")
+            
             formatted_msg = {
                 'role': msg['role'],
                 'content': msg.get('content')
