@@ -9,18 +9,20 @@ import json
 from typing import List, Dict, Any, Optional, Tuple
 from omnigen.utils.logger import setup_logger
 
-# Pydantic is REQUIRED for production - fail if not available
+# Initialize logger FIRST
+logger = setup_logger()
+
+# Pydantic is REQUIRED for production validation
 try:
     from pydantic import BaseModel, validator, Field
     from pydantic import ValidationError as PydanticValidationError
 except ImportError as e:
+    logger.error("Pydantic is REQUIRED. Install with: pip install omnigen-usf")
     raise ImportError(
-        "Pydantic is required for OmniGen. Install with: pip install pydantic>=2.0.0"
+        "Pydantic >= 2.0.0 is required for OmniGen.\n"
+        "Install with: pip install omnigen-usf"
     ) from e
 
-logger = setup_logger()
-
-
 class Message(BaseModel):
     """Message schema with full tool calling support."""
     role: str = Field(..., description="Message role")
