@@ -315,9 +315,10 @@ class ConversationExtensionConfigBuilder:
         self,
         enabled: bool = True,
         checkpoint_file: Optional[str] = None,
-        auto_save_frequency: int = 100,
         validate_input_hash: bool = True,
-        resume_mode: str = 'auto'
+        resume_mode: str = 'auto',
+        batch_save_items: int = 100,
+        batch_save_seconds: int = 10
     ) -> 'ConversationExtensionConfigBuilder':
         """
         Set checkpoint configuration.
@@ -325,9 +326,10 @@ class ConversationExtensionConfigBuilder:
         Args:
             enabled: Enable checkpoint/resume functionality
             checkpoint_file: Path to checkpoint file (auto-generated if None)
-            auto_save_frequency: Save checkpoint every N conversations (default: 100)
             validate_input_hash: Verify input file hasn't changed on resume
             resume_mode: 'auto' (resume if checkpoint exists), 'manual', or 'fresh' (ignore checkpoint)
+            batch_save_items: Save checkpoint every N items (default: 100)
+            batch_save_seconds: Save checkpoint every N seconds (default: 10)
         """
         workspace_id = self._data['workspace_id']
         
@@ -337,9 +339,10 @@ class ConversationExtensionConfigBuilder:
         self._data['checkpoint'] = {
             'enabled': enabled,
             'checkpoint_file': checkpoint_file,
-            'auto_save_frequency': auto_save_frequency,
             'validate_input_hash': validate_input_hash,
-            'resume_mode': resume_mode
+            'resume_mode': resume_mode,
+            'batch_save_items': batch_save_items,
+            'batch_save_seconds': batch_save_seconds
         }
         return self
     
@@ -391,15 +394,15 @@ class ConversationExtensionConfigBuilder:
     def set_quality_validation(
         self,
         enabled: bool = True,
-        max_retries: int = 0,
-        fail_on_quality_issues: bool = False,
+        max_retries: int = 3,
+        fail_on_quality_issues: bool = True,
         filter_failed_validations: bool = False,
         check_empty_content: bool = True,
         check_repeated_messages: bool = True,
         check_short_responses: bool = True,
         check_alternation: bool = True,
         check_tool_calls: bool = True,
-        min_message_length: int = 5
+        min_message_length: int = 1
     ) -> 'ConversationExtensionConfigBuilder':
         """
         Set quality validation configuration.
