@@ -2,7 +2,6 @@
 
 import random
 import re
-import time
 import threading
 from datetime import datetime
 from typing import List, Dict, Optional, Any, Tuple
@@ -16,6 +15,7 @@ from omnigen.storage.incremental_saver import IncrementalSaver
 from omnigen.utils.datetime_gen import DateTimeGenerator
 from omnigen.utils.rate_limiter import RateLimiter
 from omnigen.utils.logger import setup_logger
+from omnigen.core.exceptions import ShutdownException
 
 logger = setup_logger()
 
@@ -180,7 +180,6 @@ class ConversationGenerator:
     ) -> Dict:
         """Generate conversation using smart extension mode with error handling and tool support."""
         conversation = []
-        target_turns = 0
         
         # Token tracking
         token_usage_per_call = []
@@ -193,10 +192,49 @@ class ConversationGenerator:
                 conversation = partial_state.get('conversation', []).copy()
                 turns_completed = partial_state.get('turns_completed', 0)
                 last_role = partial_state.get('last_role', 'assistant')
+                # Get target_turns from partial state
+                target_turns = partial_state.get('target_turns', 0)
+                
+                # FIX: If target_turns is 0 (from old buggy checkpoint), recalculate it
+                if target_turns == 0:
+                    turn_range = self.config.get('generation.turn_range', {'min': 3, 'max': 8})
+                    turn_calculation = self.config.get('generation.turn_calculation', 'additional')
+                    current_turns = sum(1 for m in conversation if m.get('role') == 'user')
+                    
+                    if turn_calculation == 'total':
+                        if current_turns < turn_range['min']:
+                            target_turns = turn_range['min']
+                        elif current_turns >= turn_range['max']:
+                            target_turns = current_turns
+                        else:
+                            target_turns = random.randint(current_turns, turn_range['max'])
+                    else:
+                        additional_turns = random.randint(turn_range['min'], turn_range['max'])
+                        target_turns = current_turns + additional_turns
+                    
+                    logger.debug(f"Recalculated target_turns for conversation {conv_id}: {target_turns} (was 0 in checkpoint)")
             else:
                 conversation = base_conv['conversations'].copy()
                 last_role = base_conv.get('last_role', 'other')
                 turns_completed = sum(1 for m in conversation if m.get('role') == 'user')
+                
+                # Calculate target_turns BEFORE any API calls to ensure it's set even if errors occur
+                turn_range = self.config.get('generation.turn_range', {'min': 3, 'max': 8})
+                turn_calculation = self.config.get('generation.turn_calculation', 'additional')
+                current_turns = turns_completed
+                
+                if turn_calculation == 'total':
+                    # Total mode: total turns should be within range (but never remove existing)
+                    if current_turns < turn_range['min']:
+                        target_turns = turn_range['min']
+                    elif current_turns >= turn_range['max']:
+                        target_turns = current_turns  # Keep as is, don't remove
+                    else:
+                        target_turns = random.randint(current_turns, turn_range['max'])
+                else:
+                    # Additional mode (default): add NEW turns on top of existing
+                    additional_turns = random.randint(turn_range['min'], turn_range['max'])
+                    target_turns = current_turns + additional_turns
             
             # Handle based on last role
             if last_role == 'user':
@@ -233,31 +271,10 @@ class ConversationGenerator:
                     total_input_tokens += usage.get('input_tokens', 0)
                     total_output_tokens += usage.get('output_tokens', 0)
             
-            # Calculate additional turns to generate
-            turn_range = self.config.get('generation.turn_range', {'min': 3, 'max': 8})
-            turn_calculation = self.config.get('generation.turn_calculation', 'additional')
-            
+            # Calculate how many additional turns we need to reach target
+            # (target_turns already calculated at start of function)
             current_turns = sum(1 for m in conversation if m.get('role') == 'user')
-            
-            # If resuming, we've already started - continue to target
-            if partial_state:
-                target_turns = partial_state.get('target_turns', turn_range['max'])
-                additional_turns = max(0, target_turns - current_turns)
-            else:
-                # Calculate target for new generation
-                if turn_calculation == 'total':
-                    # Total mode: total turns should be within range (but never remove existing)
-                    if current_turns < turn_range['min']:
-                        target_turns = turn_range['min']
-                    elif current_turns >= turn_range['max']:
-                        target_turns = current_turns  # Keep as is, don't remove
-                    else:
-                        target_turns = random.randint(current_turns, turn_range['max'])
-                    additional_turns = target_turns - current_turns
-                else:
-                    # Additional mode (default): add NEW turns on top of existing
-                    additional_turns = random.randint(turn_range['min'], turn_range['max'])
-                    target_turns = current_turns + additional_turns
+            additional_turns = max(0, target_turns - current_turns)
             
             # Generate additional turns with error handling
             for turn_idx in range(additional_turns):
@@ -325,7 +342,7 @@ class ConversationGenerator:
                 if self.fail_on_quality_issues:
                     # Mark as failed
                     return {
-                        'id': conv_id,
+                        'id': base_conv.get('id', conv_id),  # PRESERVE ORIGINAL ID from base
                         'error': f'Quality validation failed: {validation_error}',
                         'conversations': final_conversation,
                         'success': False,
@@ -343,7 +360,7 @@ class ConversationGenerator:
                     logger.warning(f"Conversation {conv_id} quality validation failed: {validation_error}")
             
             result = {
-                'id': conv_id,
+                'id': base_conv.get('id', conv_id),  # PRESERVE ORIGINAL ID from base
                 'conversations': final_conversation,
                 'num_turns': sum(1 for m in final_conversation if m.get('role') == 'user'),
                 'num_messages': len(final_conversation),
@@ -369,11 +386,17 @@ class ConversationGenerator:
             
             return result
             
-        except Exception as e:
-            logger.error(f"Conversation {conv_id} failed: {e}")
+        except ShutdownException as e:
+            # Shutdown is NOT an error - handle gracefully without logging
+            logger.debug(f"Conversation {conv_id} interrupted by shutdown - will be resumed")
             
-            # Save partial progress if enabled
-            if self.save_partial_on_error and self.incremental_saver and conversation:
+            # Check if we made ANY progress beyond the base conversation
+            base_message_count = len(base_conv.get('conversations', []))
+            current_message_count = len(conversation) if conversation else 0
+            made_progress = current_message_count > base_message_count
+            
+            # Save partial progress for resume if we made progress
+            if made_progress and self.incremental_saver and conversation:
                 self._save_partial_progress(conv_id, conversation, target_turns, str(e))
             
             return {
@@ -381,7 +404,32 @@ class ConversationGenerator:
                 'error': str(e),
                 'conversations': conversation if conversation else base_conv.get('conversations', []),
                 'success': False,
-                'is_partial': True,
+                'is_partial': made_progress,  # Only partial if we added new messages
+                'shutdown_interrupted': True,
+                'generated_at': datetime.utcnow().isoformat(),
+                '_position': base_conv.get('_position', -1),
+                '_content_hash': base_conv.get('_content_hash', ''),
+                'num_turns': sum(1 for m in conversation if m.get('role') == 'user') if conversation else 0
+            }
+            
+        except Exception as e:
+            logger.error(f"Conversation {conv_id} failed: {e}")
+            
+            # Check if we made ANY progress beyond the base conversation
+            base_message_count = len(base_conv.get('conversations', []))
+            current_message_count = len(conversation) if conversation else 0
+            made_progress = current_message_count > base_message_count
+            
+            # Only save and mark as partial if we actually generated new content
+            if made_progress and self.save_partial_on_error and self.incremental_saver and conversation:
+                self._save_partial_progress(conv_id, conversation, target_turns, str(e))
+            
+            return {
+                'id': conv_id,
+                'error': str(e),
+                'conversations': conversation if conversation else base_conv.get('conversations', []),
+                'success': False,
+                'is_partial': made_progress,  # Only partial if we added new messages
                 'generated_at': datetime.utcnow().isoformat(),
                 '_position': base_conv.get('_position', -1),
                 '_content_hash': base_conv.get('_content_hash', ''),
@@ -413,7 +461,12 @@ class ConversationGenerator:
         if partial_state:
             conversation = partial_state.get('conversation', []).copy()
             turns_completed = partial_state.get('turns_completed', 0)
-            num_turns = partial_state.get('target_turns', turn_range['max'])
+            num_turns = partial_state.get('target_turns', 0)
+            
+            # FIX: If num_turns is 0 (from old buggy checkpoint), recalculate it
+            if num_turns == 0:
+                num_turns = random.randint(turn_range['min'], turn_range['max'])
+                logger.debug(f"Recalculated num_turns for legacy conversation {conv_id}: {num_turns} (was 0 in checkpoint)")
         else:
             conversation = []
             turns_completed = 0
@@ -523,11 +576,15 @@ class ConversationGenerator:
             
             return result
             
-        except Exception as e:
-            logger.error(f"Conversation {conv_id} failed: {e}")
+        except ShutdownException as e:
+            # Shutdown is NOT an error - handle gracefully without logging
+            logger.debug(f"Legacy conversation {conv_id} interrupted by shutdown - will be resumed")
             
-            # Save partial progress if enabled
-            if self.save_partial_on_error and self.incremental_saver and conversation:
+            # Check if we made ANY progress (at least 1 complete turn = 2 messages)
+            made_progress = len(conversation) >= 2 if conversation else False
+            
+            # Save partial progress for resume if we made progress
+            if made_progress and self.incremental_saver and conversation:
                 self._save_partial_progress(conv_id, conversation, num_turns, str(e))
             
             return {
@@ -535,7 +592,29 @@ class ConversationGenerator:
                 'error': str(e),
                 'conversations': conversation,
                 'success': False,
-                'is_partial': True,
+                'is_partial': made_progress,  # Only partial if we completed at least 1 turn
+                'shutdown_interrupted': True,
+                'generated_at': datetime.utcnow().isoformat(),
+                'num_turns': sum(1 for m in conversation if m.get('role') == 'user') if conversation else 0
+            }
+            
+        except Exception as e:
+            logger.error(f"Conversation {conv_id} failed: {e}")
+            
+            # Check if we made ANY progress (at least 1 complete turn = 2 messages)
+            # Legacy mode starts from scratch, so any complete turn is progress
+            made_progress = len(conversation) >= 2 if conversation else False
+            
+            # Only save and mark as partial if we generated at least one turn
+            if made_progress and self.save_partial_on_error and self.incremental_saver and conversation:
+                self._save_partial_progress(conv_id, conversation, num_turns, str(e))
+            
+            return {
+                'id': conv_id,
+                'error': str(e),
+                'conversations': conversation,
+                'success': False,
+                'is_partial': made_progress,  # Only partial if we completed at least 1 turn
                 'generated_at': datetime.utcnow().isoformat(),
                 'num_turns': sum(1 for m in conversation if m.get('role') == 'user') if conversation else 0
             }
@@ -606,6 +685,11 @@ class ConversationGenerator:
                     conversation.clear()
                     conversation.extend(new_conversation)
                     
+                except ShutdownException as regen_error:
+                    # Shutdown during regeneration - propagate without error logging
+                    logger.debug(f"Shutdown detected during regeneration of conversation {conv_id}")
+                    raise  # Re-raise to be handled by outer exception handler
+                    
                 except Exception as regen_error:
                     logger.error(f"Error regenerating conversation {conv_id}: {regen_error}")
                     return False, f"{error} (regeneration failed: {regen_error})"
@@ -651,15 +735,21 @@ class ConversationGenerator:
             # Check for shutdown before making API call
             if self.shutdown_event.is_set():
                 logger.debug("Shutdown detected - skipping user followup generation")
-                raise Exception("Shutdown requested - aborting conversation generation")
-            
-            history = "\n\n".join([f"{m['role'].upper()}: {m['content']}" for m in conversation])
+                raise ShutdownException("Shutdown requested - conversation will be resumed later")
+                
+            history = "\n\n".join([f"{m.get('role', 'unknown').upper()}: {m.get('content', '')}" for m in conversation if m.get('content')])
             prompt = self.prompts['followup_question'].format(history=history)
             
-            messages = [
-                {"role": "system", "content": "You are generating natural follow-up questions."},
-                {"role": "user", "content": prompt}
-            ]
+            # Build messages with configurable system message
+            messages = [{"role": "user", "content": prompt}]
+            
+            # Apply generation-only system message if configured
+            user_followup_config = self.generation_system_config.get('user_followup', {})
+            if user_followup_config.get('enabled', False):
+                system_content = user_followup_config.get('content', '').strip()
+                if system_content:
+                    # Prepend system message
+                    messages.insert(0, {"role": "system", "content": system_content})
             
             # Enforce rate limiting BEFORE API call (per-provider with role-specific limiting)
             provider_name = self.user_config.get('name', 'default')
@@ -769,7 +859,7 @@ class ConversationGenerator:
         # Check for shutdown before making API call
         if self.shutdown_event.is_set():
             logger.debug("Shutdown detected - skipping assistant response generation")
-            raise Exception("Shutdown requested - aborting conversation generation")
+            raise ShutdownException("Shutdown requested - conversation will be resumed later")
         
         # Apply dataset system messages (these WILL be saved)
         conversation_with_system = self._apply_system_messages(conversation, datetime_str)
