@@ -48,26 +48,23 @@ class StreamingConversationLoader:
                 logger.error("No file_path configured")
                 self.file_path = None
                 self.total_lines = 0
-                return
-            
-            try:
-                self.file_path = Path(self.file_path)
-                if not self.file_path.exists():
-                    logger.error(f"File not found: {self.file_path}")
+            else:
+                try:
+                    self.file_path = Path(self.file_path)
+                    if not self.file_path.exists():
+                        logger.error(f"File not found: {self.file_path}")
+                        self.total_lines = 0
+                    else:
+                        # Count total lines without loading
+                        try:
+                            self.total_lines = self._count_lines()
+                            logger.info(f"Streaming loader initialized: {self.total_lines} lines in {self.file_path}")
+                        except Exception as e:
+                            logger.error(f"Error counting lines: {e}")
+                            self.total_lines = 0
+                except Exception as e:
+                    logger.error(f"Error with file path: {e}")
                     self.total_lines = 0
-                    return
-            except Exception as e:
-                logger.error(f"Error with file path: {e}")
-                self.total_lines = 0
-                return
-            
-            # Count total lines without loading
-            try:
-                self.total_lines = self._count_lines()
-                logger.info(f"Streaming loader initialized: {self.total_lines} lines in {self.file_path}")
-            except Exception as e:
-                logger.error(f"Error counting lines: {e}")
-                self.total_lines = 0
                 
         except Exception as e:
             logger.critical(f"Failed to initialize StreamingConversationLoader: {e}", exc_info=True)
@@ -100,13 +97,13 @@ class StreamingConversationLoader:
     
     def stream_conversations(
         self,
-        skip_positions: Optional[Set[int]] = None
+        skip_ids: Optional[Set[str]] = None
     ) -> Iterator[Dict]:
         """
         Stream conversations one at a time (constant memory).
         
         Args:
-            skip_positions: Positions to skip (already processed)
+            skip_ids: IDs to skip (already processed)
             
         Yields:
             Conversation dict with metadata
@@ -116,34 +113,51 @@ class StreamingConversationLoader:
                 logger.error("Cannot stream - no file path")
                 return
             
-            skip_positions = skip_positions or set()
+            skip_ids = skip_ids or set()
             position = 0
             
             try:
                 with open(self.file_path, 'r', encoding='utf-8') as f:
                     for line in f:
-                        # Skip if already processed
+                        # Parse and validate line first (need ID)
                         try:
-                            if position in skip_positions:
+                            conv_data = self._parse_and_validate_line(line, position)
+                            if not conv_data:
                                 position += 1
                                 continue
-                        except Exception as e:
-                            logger.warning(f"Error checking skip position: {e}")
-                        
-                        # Parse and validate line
-                        try:
-                            conv_data = self._parse_and_validate_line(line, position)
-                            if conv_data:
-                                yield conv_data
                         except Exception as e:
                             logger.warning(f"Error processing line {position}: {e}")
+                            position += 1
+                            continue
+                        
+                        # Check if already processed (ID-based ONLY)
+                        item_id = conv_data.get('id')
+                        
+                        # Skip if ID already processed
+                        if item_id is not None and str(item_id) in skip_ids:
+                            logger.debug(f"Skipping ID '{item_id}' (already processed)")
+                            position += 1
+                            continue
+                        
+                        # Yield if not skipped
+                        try:
+                            yield conv_data
+                        except GeneratorExit:
+                            # Generator closed - cleanup and exit gracefully
+                            logger.debug(f"Stream closed at position {position}")
+                            break
                         
                         position += 1
                         
             except FileNotFoundError as e:
                 logger.error(f"File not found: {self.file_path} - {e}")
+                raise  # Re-raise to caller
+            except IOError as e:
+                logger.error(f"IO error streaming file: {e}", exc_info=True)
+                raise  # Re-raise to caller
             except Exception as e:
                 logger.error(f"Error streaming file: {e}", exc_info=True)
+                raise  # Re-raise to caller
                 
         except Exception as e:
             logger.critical(f"Critical error in stream_conversations: {e}", exc_info=True)
@@ -217,6 +231,19 @@ class StreamingConversationLoader:
             conv_data['parallel_tool_calls'] = parallel_tool_calls
             conv_data['strict'] = strict
             
+            # CRITICAL: Preserve original ID from base file
+            try:
+                original_id = data.get('id')
+                if original_id is not None:
+                    conv_data['id'] = original_id
+                else:
+                    # Fallback: use position as ID if not present
+                    logger.debug(f"Position {position}: No 'id' field, using position as fallback")
+                    conv_data['id'] = position
+            except Exception as e:
+                logger.warning(f"Position {position}: Error preserving ID - {e}")
+                conv_data['id'] = position  # Fallback
+            
             # Add metadata
             try:
                 content_hash = CheckpointManager.calculate_content_hash(conversations)
