@@ -1,11 +1,13 @@
 """Production-grade runner for text enhancement pipeline with streaming, monitoring, and error handling."""
 
 import sys
+import os
 import time
 import signal
 import uuid
 import threading
-from concurrent.futures import ThreadPoolExecutor, as_completed
+from pathlib import Path
+from concurrent.futures import ThreadPoolExecutor, as_completed, wait, FIRST_COMPLETED
 from typing import Optional
 from tqdm import tqdm
 from omnigen.pipelines.text_enhancement.config import TextEnhancementConfig
@@ -17,6 +19,14 @@ from omnigen.storage.incremental_saver import IncrementalSaver
 from omnigen.utils.rate_limiter import ProviderRateLimitManager
 from omnigen.utils.logger import setup_logger
 
+# Optional verification module
+try:
+    from omnigen.verification import DataVerifier
+    VERIFICATION_AVAILABLE = True
+except ImportError:
+    VERIFICATION_AVAILABLE = False
+    DataVerifier = None
+
 # Optional MongoDB monitoring
 try:
     from omnigen.monitoring.mongodb_monitor import MongoDBMonitor
@@ -115,7 +125,69 @@ class Runner:
             use_file_locking=True
         )
         
-        # Initialize streaming data loader
+        # STEP 1: VALIDATE BASE FILE (Check for duplicates, missing IDs, etc.)
+        self.original_base_file = config.get('base_data.file_path')
+        base_file_to_use = self.original_base_file
+        
+        if VERIFICATION_AVAILABLE:
+            try:
+                from omnigen.verification.base_file_validator import BaseFileValidator
+                
+                # Validate and clean base file
+                validator = BaseFileValidator(config)
+                base_file_to_use, validation_report = validator.validate_and_clean(base_file_to_use)
+                
+                if not validation_report.get('skipped', False):
+                    if validation_report.get('file_cleaned', False):
+                        logger.info(f"✅ Base file cleaned and validated")
+                        # Update the base file path
+                        self.original_base_file = base_file_to_use
+                    else:
+                        logger.info(f"✅ Base file validated (no cleaning needed)")
+            except Exception as e:
+                logger.warning(f"Base file validation error: {e}")
+                # Continue with original file
+        
+        # STEP 2: CHECK FOR RECHECK FILE - Priority loading
+        self.using_recheck_file = False
+        
+        if VERIFICATION_AVAILABLE:
+            base_path = Path(self.original_base_file)
+            recheck_file = base_path.parent / f"{base_path.stem}_rechecked.jsonl"
+            
+            if recheck_file.exists():
+                # Count items in recheck file
+                recheck_count = sum(1 for _ in open(recheck_file))
+                
+                logger.info("=" * 60)
+                logger.info("📋 RECHECK FILE DETECTED!")
+                logger.info("=" * 60)
+                logger.info(f"   Recheck file: {recheck_file.name}")
+                logger.info(f"   Items to reprocess: {recheck_count}")
+                logger.info(f"   These items were missing or invalid in previous run")
+                logger.info(f"\n   🔄 PRIORITY MODE: Processing recheck file FIRST")
+                logger.info(f"\n   ⚠️  CHECKPOINT DISABLED: Starting fresh for recheck items")
+                logger.info(f"   Reason: Checkpoint may be corrupted (that's why we're rechecking)")
+                logger.info("=" * 60)
+                
+                # Use recheck file as base
+                base_file_to_use = str(recheck_file)
+                self.using_recheck_file = True
+                
+                # CRITICAL: Disable checkpoint when using recheck file
+                # Reason: We're rechecking BECAUSE checkpoint might be corrupted
+                # If checkpoint was trustworthy, we wouldn't need to recheck!
+                if self.checkpoint_manager:
+                    logger.warning("⚠️  Temporarily disabling checkpoint for recheck mode")
+                    logger.warning("   Will not resume from old checkpoint (may be corrupted)")
+                    logger.warning("   Will create fresh checkpoint for recheck run")
+                    # Disable checkpoint manager during recheck
+                    self.checkpoint_manager = None
+                
+                # Update config temporarily
+                config.config['base_data']['file_path'] = base_file_to_use
+        
+        # Initialize streaming data loader (with potentially modified config)
         self.data_loader = StreamingTextLoader(config, self.checkpoint_manager)
         
         # Track for graceful shutdown
@@ -143,7 +215,7 @@ class Runner:
             else:
                 # Second Ctrl+C = force immediate exit
                 logger.error("\n⚠️  FORCE EXIT - Shutting down immediately without saving!")
-                sys.exit(1)
+                os._exit(1)  # Use os._exit instead of sys.exit to avoid SystemExit exception in signal handler
         
         signal.signal(signal.SIGINT, signal_handler)
         signal.signal(signal.SIGTERM, signal_handler)
@@ -161,12 +233,16 @@ class Runner:
             
             # Add to checkpoint (in-memory update, actual save is batched)
             if self.checkpoint_manager:
+                # Extract item_id from result for ID-based tracking
+                item_id = result.get('id', str(position))  # Fallback to position if no ID
+                
                 self.checkpoint_manager.add_processed(
                     position,
                     content_hash,
                     status,
                     result,
-                    save_checkpoint=False  # Batch save
+                    save_checkpoint=False,  # Batch save
+                    item_id=item_id  # NEW: Pass item_id for ID-based tracking
                 )
         except Exception as e:
             logger.error(f"Error in async save: {e}", exc_info=True)
@@ -194,9 +270,9 @@ class Runner:
             
             # ConcurrencyLimiter has 'active_calls' and 'max_concurrent'
             if 'active_calls' in first_limiter_stats and 'max_concurrent' in first_limiter_stats:
+                # Show only actual active calls (not capacity)
                 total_active = sum(stats.get('active_calls', 0) for stats in all_stats.values())
-                total_max = sum(stats.get('max_concurrent', 0) for stats in all_stats.values())
-                return f"Active:{total_active}/{total_max}"
+                return f"API Calls:{total_active}"
             
             # RateLimiter has 'current_rpm'
             elif 'current_rpm' in first_limiter_stats:
@@ -365,19 +441,31 @@ class Runner:
             failed = progress['failed']
             skipped = progress['skipped']
         
-        # Get already processed positions
-        skip_positions = set()
+        # Get already processed IDs (ID-based ONLY)
+        skip_ids = set()
         if self.checkpoint_manager:
-            skip_positions = self.checkpoint_manager.get_processed_positions()
-            logger.info(f"Skipping {len(skip_positions)} already processed positions")
+            skip_ids = self.checkpoint_manager.get_processed_ids()
+            if skip_ids:
+                logger.info(f"📋 Skipping {len(skip_ids)} already processed IDs")
         
         pbar = tqdm(
             total=num_texts,
-            desc="Enhancing",
-            unit=" text",
+            desc="📝 Enhancing",
+            unit="text",
+            bar_format='{l_bar}{bar}| {n_fmt}/{total_fmt} [{elapsed}<{remaining}, {rate_fmt}] {postfix}',
             colour='cyan',
-            ncols=140,
-            initial=complete + partial + failed + skipped
+            ncols=None,        # Auto-detect terminal width to prevent truncation
+            dynamic_ncols=True,  # Allow dynamic terminal width updates
+            initial=complete + partial + failed + skipped,
+            leave=True,        # Keep final bar visible
+            position=0,        # Always update at position 0 (no scrolling)
+            miniters=1,        # Update every iteration
+            mininterval=0.1    # Minimum 0.1s between updates (smooth but not excessive)
+        )
+        
+        # Set initial status display
+        pbar.set_postfix_str(
+            f"✓{complete} ⚠{partial} ✗{failed} ~{skipped} | Active:0/{num_workers} Queue:0 {self._get_rate_limit_metric()}"
         )
         
         # Separate executor for async I/O operations (saving, checkpointing)
@@ -387,11 +475,22 @@ class Runner:
         with ThreadPoolExecutor(max_workers=num_workers, thread_name_prefix="Worker") as executor:
             futures = {}
             io_futures = []  # Track I/O operations
-            submitted = 0
-            text_stream = self.data_loader.stream_texts(skip_positions=skip_positions)
+            
+            # CRITICAL FIX: Start submitted from checkpoint progress on resume
+            if self.checkpoint_manager:
+                progress = self.checkpoint_manager.get_progress_summary()
+                submitted = progress['total_processed']
+                logger.info(f"🔄 Resume: Starting submitted counter at {submitted} (already processed)")
+            else:
+                submitted = 0
+            
+            # Pass skip_ids for ID-based resume
+            text_stream = self.data_loader.stream_texts(skip_ids=skip_ids)
             
             # Initial batch submission - keep executor saturated
-            initial_batch = min(num_workers * 2, num_texts)  # 2x workers for saturation
+            remaining = num_texts - submitted
+            initial_batch = min(num_workers * 2, remaining)
+            logger.info(f"📤 Submitting initial batch of {initial_batch} texts (remaining: {remaining})")
             for _ in range(initial_batch):
                 if self.shutdown_event.is_set():
                     break
@@ -414,8 +513,7 @@ class Runner:
                     logger.error(f"Error submitting text: {e}")
             
             # Process results with dynamic submission (keep workers saturated)
-            batch_count = 0
-            auto_save_freq = self.config.get('checkpoint.auto_save_frequency', 100)
+            # Note: Checkpoint saves are handled automatically by checkpoint.py batch_save logic
             
             # Emergency shutdown: cancel all pending futures and save checkpoint
             if self.shutdown_event.is_set():
@@ -454,7 +552,7 @@ class Runner:
                 shutdown_start = time.time()
                 completed_after_shutdown = 0
                 
-                for future in as_completed(futures, timeout=5):
+                for future in as_completed(list(futures.keys()), timeout=5):
                     try:
                         elapsed = time.time() - shutdown_start
                         remaining = max(0, 5 - elapsed)
@@ -482,15 +580,22 @@ class Runner:
                 total_shutdown_time = time.time() - cancel_start
                 logger.info(f"✓ Completed {completed_after_shutdown} in-flight requests")
                 logger.info(f"✓ Total shutdown time: {total_shutdown_time:.2f}s")
+                
+                # Enhancement 6: Flush any pending checkpoint before exit
+                if self.checkpoint_manager:
+                    self.checkpoint_manager.flush_checkpoint()
+                
                 logger.warning("🛑 SHUTDOWN COMPLETE - Exiting now")
                 
                 pbar.set_postfix_str(f"✓ Shutdown complete in {total_shutdown_time:.2f}s")
                 pbar.close()
                 
-                # Force exit
-                sys.exit(0)
+                # Clean exit after graceful shutdown
+                os._exit(0)  # Use os._exit to avoid SystemExit exception
             
-            for future in as_completed(futures):
+            # CRITICAL FIX: Use while loop with wait() to handle dynamic futures
+            while futures or submitted < num_texts:
+                # Check for shutdown
                 if self.shutdown_event.is_set():
                     pending_count = sum(1 for f in futures if not f.done())
                     in_flight_count = len([f for f in futures if not f.done() and not f.cancelled()])
@@ -553,88 +658,107 @@ class Runner:
                     total_shutdown_time = time.time() - cancel_start
                     logger.info(f"✓ Completed {completed_after_shutdown} in-flight requests")
                     logger.info(f"✓ Total shutdown time: {total_shutdown_time:.2f}s")
+                    
+                    # Enhancement 6: Flush any pending checkpoint before exit
+                    if self.checkpoint_manager:
+                        self.checkpoint_manager.flush_checkpoint()
+                    
                     logger.warning("🛑 SHUTDOWN COMPLETE - Exiting now")
                     
                     pbar.set_postfix_str(f"✓ Shutdown complete in {total_shutdown_time:.2f}s")
                     pbar.close()
-                    sys.exit(0)
+                    os._exit(0)  # Use os._exit to avoid SystemExit exception
                 
-                # IMMEDIATELY submit next item to keep workers saturated
-                # Do this BEFORE processing result to minimize idle time
-                if submitted < num_texts and not self.shutdown_event.is_set():
+                # Wait for at least one future to complete
+                if not futures:
+                    if submitted >= num_texts:
+                        break  # All done!
+                    continue  # Need to submit more
+                
+                # Use wait() to get completed futures (sees newly added futures!)
+                done_futures, _ = wait(futures.keys(), return_when=FIRST_COMPLETED, timeout=0.1)
+                
+                # Process each completed future
+                for future in done_futures:
+                    # IMMEDIATELY submit next item to keep workers saturated
+                    if submitted < num_texts and not self.shutdown_event.is_set():
+                        try:
+                            text_data = next(text_stream)
+                            position = text_data.get('_position', -1)
+                            content_hash = text_data.get('_content_hash', '')
+                            
+                            new_future = executor.submit(
+                                self._process_text_with_retry,
+                                text_data,
+                                submitted
+                            )
+                            futures[new_future] = (submitted, position, content_hash)
+                            submitted += 1
+                        except StopIteration:
+                            pass  # No more texts to process
+                        except Exception as e:
+                            logger.error(f"Error submitting next text: {e}")
+                    
+                    # Now process the completed result
+                    text_id, position, content_hash = futures[future]
+                    
                     try:
-                        text_data = next(text_stream)
-                        position = text_data.get('_position', -1)
-                        content_hash = text_data.get('_content_hash', '')
+                        result = future.result()
                         
-                        new_future = executor.submit(
-                            self._process_text_with_retry,
-                            text_data,
-                            submitted
-                        )
-                        futures[new_future] = (submitted, position, content_hash)
-                        submitted += 1
-                    except StopIteration:
-                        pass  # No more texts to process
-                    except Exception as e:
-                        logger.error(f"Error submitting next text: {e}")
-                
-                # Now process the completed result
-                text_id, position, content_hash = futures[future]
-                
-                try:
-                    result = future.result()
-                    
-                    # Skip if marked as skipped
-                    if result.get('skipped'):
-                        skipped += 1
-                        pbar.update(1)
-                        continue
-                    
-                    # Determine status
-                    status = 'failed'
-                    if result.get('success'):
-                        complete += 1
-                        status = 'completed'
-                    else:
-                        failed += 1
+                        # Skip if marked as skipped
+                        if result.get('skipped'):
+                            skipped += 1
+                            pbar.update(1)
+                            del futures[future]
+                            continue
+                        
+                        # Determine status
                         status = 'failed'
-                    
-                    # Aggregate token usage
-                    if 'tokens' in result:
-                        tokens = result['tokens']
-                        total_input_tokens += tokens.get('input_tokens', 0)
-                        total_output_tokens += tokens.get('output_tokens', 0)
-                    
-                    # ASYNC I/O: Submit saving to separate executor (non-blocking!)
-                    # This ensures LLM workers never wait for disk I/O
-                    io_future = io_executor.submit(
-                        self._async_save_result,
-                        result,
-                        status,
-                        position,
-                        content_hash
-                    )
-                    io_futures.append(io_future)
-                    
-                    # Batch checkpoint updates (minimal blocking)
-                    batch_count += 1
-                    if batch_count >= auto_save_freq:
-                        # Also async to avoid blocking
-                        io_executor.submit(self._async_save_checkpoint)
-                        batch_count = 0
-                    
-                    # Update progress immediately (fast, non-blocking)
-                    metric_str = self._get_rate_limit_metric()
-                    pbar.update(1)
-                    pbar.set_postfix_str(
-                        f"✓{complete} ⚠{partial} ✗{failed} ~{skipped} | {metric_str}"
-                    )
-                    
-                except Exception as e:
-                    logger.error(f"Error processing text {text_id}: {e}")
-                    failed += 1
-                    pbar.update(1)
+                        if result.get('success'):
+                            complete += 1
+                            status = 'completed'
+                        else:
+                            failed += 1
+                            status = 'failed'
+                        
+                        # Aggregate token usage
+                        if 'tokens' in result:
+                            tokens = result['tokens']
+                            total_input_tokens += tokens.get('input_tokens', 0)
+                            total_output_tokens += tokens.get('output_tokens', 0)
+                        
+                        # ASYNC I/O: Submit saving to separate executor (non-blocking!)
+                        # This ensures LLM workers never wait for disk I/O
+                        io_future = io_executor.submit(
+                            self._async_save_result,
+                            result,
+                            status,
+                            position,
+                            content_hash
+                        )
+                        io_futures.append(io_future)
+                        
+                        # Note: Checkpoint saves are now handled by checkpoint.py batch_save logic
+                        # (triggered automatically in update_checkpoint based on batch_save_items/seconds)
+                        
+                        # Update progress immediately (fast, non-blocking)
+                        metric_str = self._get_rate_limit_metric()
+                        active_workers = min(len(futures), num_workers)
+                        queue_size = max(0, len(futures) - num_workers)
+                        pbar.update(1)
+                        pbar.set_postfix_str(
+                            f"✓{complete} ⚠{partial} ✗{failed} ~{skipped} | Active:{active_workers}/{num_workers} Queue:{queue_size} {metric_str}"
+                        )
+                        
+                        # Remove from futures dict after processing
+                        del futures[future]
+                        
+                    except Exception as e:
+                        logger.error(f"Error processing text {text_id}: {e}")
+                        failed += 1
+                        pbar.update(1)
+                        # Remove from futures dict even on error
+                        del futures[future]
             
             # Wait for all I/O operations to complete
             logger.info("Waiting for I/O operations to complete...")
@@ -646,6 +770,7 @@ class Runner:
             
             # Final checkpoint save
             if self.checkpoint_manager:
+                self.checkpoint_manager.flush_checkpoint()  # Enhancement 6: Flush any pending saves
                 self.checkpoint_manager._save_checkpoint()
                 logger.info("✓ Final checkpoint saved")
             
@@ -654,24 +779,42 @@ class Runner:
         
         pbar.close()
         
-        
         # Summary
         total_time = time.time() - start_time
-        total = complete + partial + failed
+        total_processed = complete + partial + failed
         
-        print("\n" + "="*60)
-        print("ENHANCEMENT COMPLETE")
-        print("="*60)
-        print(f"✓ Complete:  {complete:>5}")
-        print(f"⚠ Partial:   {partial:>5}")
-        print(f"✗ Failed:    {failed:>5}")
-        print(f"~ Skipped:   {skipped:>5}")
-        print(f"━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━")
-        print(f"💾 Saved:   {complete + partial:>5}  ({(complete+partial)/total*100:.1f}% of processed)" if total > 0 else "💾 Saved:   0")
-        print(f"⏱  Time:    {total_time/60:>5.1f} min")
-        if total > 0:
-            print(f"⚡ Speed:   {total/total_time:>5.2f} text/s")
-        print("="*60)
+        logger.info("\n" + "="*70)
+        logger.info("🎉 ENHANCEMENT COMPLETE")
+        logger.info("="*70)
+        logger.info("\n📊 Status Breakdown:")
+        logger.info(f"  ✓ Complete:  {complete:>6}  (Successfully enhanced)")
+        logger.info(f"  ⚠ Partial:   {partial:>6}  (Enhancement failed, saved original)")
+        logger.info(f"  ✗ Failed:    {failed:>6}  (Enhancement failed completely)")
+        logger.info(f"  ~ Skipped:   {skipped:>6}  (Already processed in checkpoint)")
+        logger.info("  " + "─"*66)
+        logger.info(f"  📊 Total Processed:  {total_processed:>6}  (complete + partial + failed)")
+        
+        logger.info("\n💾 Output Summary:")
+        logger.info(f"  Saved to disk:       {complete + partial:>6}  texts")
+        if total_processed > 0:
+            success_rate = (complete / total_processed * 100) if total_processed > 0 else 0
+            save_rate = ((complete + partial) / total_processed * 100) if total_processed > 0 else 0
+            logger.info(f"  Success rate:        {success_rate:>6.1f}%  (complete/processed)")
+            logger.info(f"  Save rate:           {save_rate:>6.1f}%  (saved/processed)")
+        
+        logger.info("\n⏱️  Performance:")
+        logger.info(f"  Total time:          {total_time/60:>6.1f}  minutes")
+        if total_processed > 0:
+            logger.info(f"  Processing speed:    {total_processed/total_time:>6.2f}  text/s")
+            logger.info(f"  Avg time per text:   {total_time/total_processed:>6.2f}  seconds")
+        
+        if total_input_tokens > 0 or total_output_tokens > 0:
+            logger.info("\n🎫 Token Usage:")
+            logger.info(f"  Input tokens:        {total_input_tokens:>6,}")
+            logger.info(f"  Output tokens:       {total_output_tokens:>6,}")
+            logger.info(f"  Total tokens:        {total_input_tokens + total_output_tokens:>6,}")
+        
+        logger.info("="*70)
         
         if self.checkpoint_manager:
             print(f"\n📊 Checkpoint: {self.checkpoint_manager.checkpoint_path}")
@@ -702,6 +845,51 @@ class Runner:
                 print(f"   Avg Input:     {avg_input:>12,.0f}")
                 print(f"   Avg Output:    {avg_output:>12,.0f}")
                 print(f"   Avg Total:     {avg_total:>12,.0f}")
+        
+        # DATA VERIFICATION: Check for missing/invalid items
+        if VERIFICATION_AVAILABLE:
+            try:
+                print(f"\n")
+                logger.info("="*60)
+                logger.info("STARTING DATA VERIFICATION")
+                logger.info("="*60)
+                
+                verifier = DataVerifier(self.config.to_dict(), pipeline_type='text')
+                
+                # CRITICAL: Always verify against ORIGINAL base file
+                # (not recheck file, even if we were using one for input)
+                base_file = self.original_base_file
+                output_dir = self.config.get('output.output_dir')
+                
+                recheck_file, num_to_recheck = verifier.verify_and_create_recheck(
+                    base_file=base_file,
+                    output_dir=output_dir
+                )
+                
+                if recheck_file:
+                    print(f"\n⚠️  DATA QUALITY ALERT")
+                    print(f"   Found {num_to_recheck} items that need reprocessing")
+                    print(f"   Recheck file created: {Path(recheck_file).name}")
+                    print(f"\n💡 Next run will automatically process recheck file first")
+                else:
+                    print(f"\n✅ DATA VERIFICATION PASSED")
+                    print(f"   All items processed correctly")
+                    
+                    # If we were using a recheck file and everything passed, delete it
+                    if self.using_recheck_file:
+                        try:
+                            base_path = Path(self.original_base_file)
+                            old_recheck = base_path.parent / f"{base_path.stem}_rechecked.jsonl"
+                            if old_recheck.exists():
+                                old_recheck.unlink()
+                                print(f"\n🗑️  Cleaned up recheck file (all items now processed)")
+                                logger.info(f"✓ Deleted recheck file: {old_recheck.name}")
+                        except Exception as cleanup_error:
+                            logger.warning(f"Could not delete recheck file: {cleanup_error}")
+                    
+            except Exception as e:
+                logger.error(f"Error during data verification: {e}")
+                logger.debug("Verification error details:", exc_info=True)
     
     def _process_text_with_retry(
         self,
