@@ -137,11 +137,14 @@ class OpenAIProvider(BaseLLMProvider):
         model: Optional[str] = None,
         temperature: float = 0.7,
         max_tokens: Optional[int] = None,
+        tools: Optional[List[Dict]] = None,
+        tool_choice: Optional[str] = None,
+        parallel_tool_calls: Optional[bool] = None,
         return_usage: bool = False,
         **kwargs
     ) -> Any:
         """
-        Generate chat completion with comprehensive error handling.
+        Generate chat completion with comprehensive error handling and tool support.
         
         Automatically uses streaming if use_streaming=True in config.
         
@@ -150,11 +153,15 @@ class OpenAIProvider(BaseLLMProvider):
             model: Model override
             temperature: Sampling temperature
             max_tokens: Maximum tokens
+            tools: List of tool definitions (OpenAI format)
+            tool_choice: Tool choice strategy ("auto", "none", "required", or specific tool)
+            parallel_tool_calls: Whether to allow parallel tool calls
             return_usage: If True, return tuple of (content, usage_dict)
             **kwargs: Additional parameters
             
         Returns:
             Generated text, or tuple of (content, usage_dict) if return_usage=True
+            Can also return tuple of (content, tool_calls) if model makes tool calls
             
         Raises:
             RateLimitError: Rate limit exceeded
@@ -165,7 +172,9 @@ class OpenAIProvider(BaseLLMProvider):
         # Use streaming mode if configured
         if self.use_streaming:
             return self._stream_and_collect(
-                messages, model, temperature, max_tokens, return_usage, **kwargs
+                messages, model, temperature, max_tokens, return_usage,
+                tools=tools, tool_choice=tool_choice, parallel_tool_calls=parallel_tool_calls,
+                **kwargs
             )
         
         # Non-streaming implementation
@@ -173,17 +182,48 @@ class OpenAIProvider(BaseLLMProvider):
             model = model or self.model
             formatted_messages = self._prepare_messages(messages)
             
+            # Build API parameters
+            api_params = {
+                'model': model,
+                'messages': formatted_messages,
+                'temperature': temperature,
+                **kwargs
+            }
+            
+            if max_tokens is not None:
+                api_params['max_tokens'] = max_tokens
+            
+            # Add tool parameters if tools are provided
+            if tools:
+                api_params['tools'] = tools
+                if tool_choice:
+                    api_params['tool_choice'] = tool_choice
+                if parallel_tool_calls is not None:
+                    api_params['parallel_tool_calls'] = parallel_tool_calls
+            
             for attempt in range(self.max_retries):
                 try:
-                    response = self.client.chat.completions.create(
-                        model=model,
-                        messages=formatted_messages,
-                        temperature=temperature,
-                        max_tokens=max_tokens,
-                        **kwargs
-                    )
+                    response = self.client.chat.completions.create(**api_params)
                     
-                    content = response.choices[0].message.content.strip()
+                    message = response.choices[0].message
+                    content = message.content or ""
+                    if content:
+                        content = content.strip()
+                    
+                    # Check for tool calls in response
+                    result_tool_calls = None
+                    if hasattr(message, 'tool_calls') and message.tool_calls:
+                        result_tool_calls = [
+                            {
+                                'id': tc.id,
+                                'type': tc.type,
+                                'function': {
+                                    'name': tc.function.name,
+                                    'arguments': tc.function.arguments
+                                }
+                            }
+                            for tc in message.tool_calls
+                        ]
                     
                     if return_usage:
                         # Extract token usage from response
@@ -194,8 +234,16 @@ class OpenAIProvider(BaseLLMProvider):
                                 'output_tokens': getattr(response.usage, 'completion_tokens', 0),
                                 'total_tokens': getattr(response.usage, 'total_tokens', 0)
                             }
+                        
+                        # Return different formats based on tool_calls presence
+                        if result_tool_calls:
+                            return (content, result_tool_calls), usage
                         return content, usage
                     
+                    # Return tool_calls if present
+                    if result_tool_calls:
+                        return content, result_tool_calls
+                    
                     return content
                     
                 except Exception as e:
