@@ -38,6 +38,7 @@ class IncrementalSaver:
         partial_file: str,
         failed_file: str,
         rejected_file: str = None,
+        aborted_file: str = None,
         use_file_locking: bool = True
     ):
         """
@@ -48,6 +49,7 @@ class IncrementalSaver:
             partial_file: File for partial conversations
             failed_file: File for failed conversations
             rejected_file: File for rejected items (validation failures)
+            aborted_file: File for aborted items (shutdown interrupted)
             use_file_locking: Use file locking for concurrent writes
         """
         try:
@@ -55,6 +57,7 @@ class IncrementalSaver:
             self.partial_file = Path(partial_file)
             self.failed_file = Path(failed_file)
             self.rejected_file = Path(rejected_file) if rejected_file else self.failed_file.parent / 'rejected.jsonl'
+            self.aborted_file = Path(aborted_file) if aborted_file else self.failed_file.parent / 'aborted.jsonl'
             self.use_file_locking = use_file_locking
             
             # Thread-safe writing
@@ -62,7 +65,7 @@ class IncrementalSaver:
             
             # Ensure directories exist
             try:
-                for filepath in [self.output_file, self.partial_file, self.failed_file, self.rejected_file]:
+                for filepath in [self.output_file, self.partial_file, self.failed_file, self.rejected_file, self.aborted_file]:
                     filepath.parent.mkdir(parents=True, exist_ok=True)
             except Exception as e:
                 logger.error(f"Failed to create directories: {e}", exc_info=True)
@@ -77,6 +80,7 @@ class IncrementalSaver:
                 self.partial_file = Path("partial.jsonl")
                 self.failed_file = Path("failed.jsonl")
                 self.rejected_file = Path("rejected.jsonl")
+                self.aborted_file = Path("aborted.jsonl")
                 self.use_file_locking = False
                 self.write_lock = Lock()
             except:
@@ -88,7 +92,7 @@ class IncrementalSaver:
         status: str = 'completed'
     ) -> bool:
         """
-        Save conversation immediately.
+        Save conversation immediately with thread-safe guarantees.
         
         Args:
             conversation: Conversation data
@@ -98,34 +102,33 @@ class IncrementalSaver:
             True if saved successfully, False otherwise
         """
         try:
-            # Add status metadata
-            conv_with_status = conversation.copy()
-            conv_with_status['status'] = status
-            conv_with_status['is_complete'] = (status == 'completed')
-            
-            # Determine target file
-            if status == 'completed':
-                target_file = self.output_file
-            elif status == 'partial':
-                target_file = self.partial_file
-            elif status == 'rejected':
-                target_file = self.rejected_file
-            else:  # failed
-                target_file = self.failed_file
-            
-            # Atomic write with locking
-            try:
-                with self.write_lock:
-                    success = self._atomic_append(target_file, conv_with_status)
-                    if success:
-                        logger.debug(f"Saved conversation {conversation.get('id')} as {status}")
-                        return True
-                    else:
-                        logger.warning(f"Failed to save conversation {conversation.get('id')}")
-                        return False
-            except Exception as e:
-                logger.error(f"Error in save_conversation lock: {e}", exc_info=True)
-                return False
+            # CRITICAL: Always use lock to prevent data corruption during shutdown
+            with self.write_lock:
+                # Add status metadata
+                conv_with_status = conversation.copy()
+                conv_with_status['status'] = status
+                conv_with_status['is_complete'] = (status == 'completed')
+                
+                # Determine target file
+                if status == 'completed':
+                    target_file = self.output_file
+                elif status == 'partial':
+                    target_file = self.partial_file
+                elif status == 'rejected':
+                    target_file = self.rejected_file
+                elif status == 'aborted':
+                    target_file = self.aborted_file
+                else:  # failed
+                    target_file = self.failed_file
+                
+                # Atomic write (already inside lock)
+                success = self._atomic_append(target_file, conv_with_status)
+                if success:
+                    logger.debug(f"Saved conversation {conversation.get('id')} as {status}")
+                    return True
+                else:
+                    logger.warning(f"Failed to save conversation {conversation.get('id')}")
+                    return False
                 
         except Exception as e:
             logger.error(f"Failed to save conversation: {e}", exc_info=True)
@@ -207,8 +210,8 @@ class IncrementalSaver:
                             finally:
                                 try:
                                     fcntl.flock(f.fileno(), fcntl.LOCK_UN)
-                                except:
-                                    pass
+                                except Exception as unlock_error:
+                                    logger.debug(f"Error unlocking file: {unlock_error}")
                         except Exception as e:
                             logger.error(f"Error with file lock: {e}", exc_info=True)
                             # Try without lock
@@ -216,7 +219,8 @@ class IncrementalSaver:
                                 f.write(json.dumps(data, ensure_ascii=False) + '\n')
                                 f.flush()
                                 return True
-                            except:
+                            except Exception as write_error:
+                                logger.error(f"Error writing without lock: {write_error}")
                                 return False
                                 
                 except ImportError:
@@ -249,8 +253,8 @@ class IncrementalSaver:
                 f.flush()
                 try:
                     os.fsync(f.fileno())
-                except:
-                    pass  # fsync may not be available on all systems
+                except Exception as fsync_error:
+                    logger.debug(f"fsync not available: {fsync_error}")  # Not critical
             return True
         except Exception as e:
             logger.error(f"Error in simple append: {e}", exc_info=True)
