@@ -8,7 +8,7 @@ and per-provider rate management.
 import time
 import threading
 from collections import deque
-from typing import Optional, Dict
+from typing import Optional, Dict, Union
 from omnigen.utils.logger import setup_logger
 
 logger = setup_logger()
@@ -310,6 +310,129 @@ class RateLimiter:
             }
 
 
+class ConcurrencyLimiter:
+    """
+    Simple concurrency limiter using semaphore.
+    Limits the number of concurrent API calls.
+    
+    This is simpler and more intuitive than RPM-based rate limiting.
+    Use when you want to control concurrent calls rather than requests per minute.
+    
+    Features:
+    - Thread-safe semaphore-based limiting
+    - Acquire/release pattern for concurrent control
+    - Statistics tracking
+    - Configurable timeout on acquire
+    
+    Example:
+        limiter = ConcurrencyLimiter(max_concurrent=50)
+        if limiter.acquire(timeout=120):
+            try:
+                # Make API call
+                response = api_call()
+            finally:
+                limiter.release()
+    """
+    
+    def __init__(self, max_concurrent: int, name: str = "default"):
+        """
+        Initialize concurrency limiter.
+        
+        Args:
+            max_concurrent: Maximum number of concurrent calls allowed
+            name: Identifier for this limiter (e.g., 'openai_user_followup')
+        """
+        try:
+            self.max_concurrent = max(1, max_concurrent)  # Minimum 1
+            self.name = name
+            self.semaphore = threading.Semaphore(self.max_concurrent)
+            self.active_count = 0
+            self.total_requests = 0
+            self.lock = threading.Lock()
+            
+            logger.debug(f"ConcurrencyLimiter initialized for {name}: max {self.max_concurrent} concurrent calls")
+        except Exception as e:
+            logger.error(f"Failed to initialize ConcurrencyLimiter: {e}", exc_info=True)
+            # Set safe defaults
+            self.max_concurrent = 1
+            self.name = name
+            self.semaphore = threading.Semaphore(1)
+            self.active_count = 0
+            self.total_requests = 0
+            self.lock = threading.Lock()
+    
+    def acquire(self, timeout: Optional[float] = 120) -> bool:
+        """
+        Acquire permission to make a request (blocking until slot available).
+        
+        Args:
+            timeout: Maximum time to wait in seconds (default: 120)
+            
+        Returns:
+            True if acquired, False if timeout
+        """
+        try:
+            acquired = self.semaphore.acquire(timeout=timeout)
+            if acquired:
+                with self.lock:
+                    self.active_count += 1
+                    self.total_requests += 1
+            else:
+                logger.warning(f"Concurrency limit timeout after {timeout}s for {self.name}")
+            return acquired
+        except Exception as e:
+            logger.error(f"Error acquiring concurrency slot: {e}", exc_info=True)
+            # On error, allow request
+            return True
+    
+    def release(self):
+        """Release the semaphore slot after request completes."""
+        try:
+            self.semaphore.release()
+            with self.lock:
+                self.active_count -= 1
+        except Exception as e:
+            logger.error(f"Error releasing concurrency slot: {e}", exc_info=True)
+    
+    def get_current_rpm(self) -> int:
+        """
+        Get current RPM (returns 0 for concurrency limiters).
+        
+        Note: Concurrency limiters don't track RPM, they track concurrent calls.
+        This method exists for compatibility with ProviderRateLimitManager.get_rpm().
+        
+        Returns:
+            0 (concurrency limiters don't measure RPM)
+        """
+        return 0
+    
+    def record_request(self):
+        """
+        Record a request (no-op for concurrency limiters).
+        
+        Note: Concurrency limiters use semaphores and don't need explicit request recording.
+        This method exists for compatibility with the RateLimiter interface.
+        """
+        pass
+    
+    def get_stats(self) -> dict:
+        """Get current concurrency statistics."""
+        try:
+            with self.lock:
+                return {
+                    'max_concurrent': self.max_concurrent,
+                    'active_calls': self.active_count,
+                    'total_requests': self.total_requests,
+                    'available_slots': self.max_concurrent - self.active_count
+                }
+        except Exception as e:
+            logger.error(f"Error getting stats: {e}", exc_info=True)
+            return {
+                'provider': self.name,
+                'error': str(e)
+            }
+
+
 class ProviderRateLimitManager:
     """
     Manage rate limiters for multiple providers.
@@ -322,7 +445,7 @@ class ProviderRateLimitManager:
     def __init__(self):
         """Initialize rate limit manager."""
         try:
-            self.limiters: Dict[str, RateLimiter] = {}
+            self.limiters: Dict[str, Union[RateLimiter, ConcurrencyLimiter]] = {}
             self.lock = threading.Lock()
             logger.info("ProviderRateLimitManager initialized")
         except Exception as e:
@@ -335,30 +458,41 @@ class ProviderRateLimitManager:
         provider_name: str,
         provider_config: Optional[Dict] = None,
         role: Optional[str] = None
-    ) -> RateLimiter:
+    ) -> Union[RateLimiter, ConcurrencyLimiter]:
         """
         Get or create rate limiter for provider.
         
+        Supports two types of limiting:
+        1. Concurrency-based (max_concurrent_calls) - simpler, limits concurrent requests
+        2. RPM-based (rate_limit_rpm) - traditional rate limiting
+        
+        If both are specified, concurrency limiting takes precedence.
+        
         Args:
             provider_name: Provider name (e.g., 'openai', 'anthropic')
             provider_config: Optional provider configuration dict with:
-                - rate_limit_rpm: Custom RPM limit (overrides defaults)
+                - max_concurrent_calls: Maximum concurrent API calls (creates ConcurrencyLimiter)
+                - rate_limit_rpm: Custom RPM limit (creates RateLimiter, overrides defaults)
                 - rate_limit_shared_key: Shared key for pooling limits across roles
                 - model: Model name for auto-detection
             role: Optional role name for independent rate limiters per role
             
         Returns:
-            RateLimiter instance
+            ConcurrencyLimiter or RateLimiter instance
             
         Examples:
+            # Concurrency limiting (recommended for simplicity)
+            limiter = manager.get_limiter('openai', {'max_concurrent_calls': 50}, 'user_followup')
+            # → Creates: openai_user_followup with max 50 concurrent calls
+            
+            # RPM-based limiting
+            limiter = manager.get_limiter('openai', {'rate_limit_rpm': 300}, 'user_followup')
+            # → Creates: openai_user_followup with 300 RPM
+            
             # Auto-detect from model
             limiter = manager.get_limiter('openai', {'model': 'gpt-4o-mini'}, 'user_followup')
             # → Creates: openai_gpt-4o-mini_user_followup with 500 RPM
             
-            # Custom RPM
-            limiter = manager.get_limiter('openai', {'rate_limit_rpm': 300}, 'user_followup')
-            # → Creates: openai_user_followup with 300 RPM
-            
             # Shared key
             limiter = manager.get_limiter('openai', {'rate_limit_shared_key': 'my_key'})
             # → Creates: my_key with auto-detected RPM (reused across roles)
@@ -372,10 +506,28 @@ class ProviderRateLimitManager:
                 if limiter_key in self.limiters:
                     return self.limiters[limiter_key]
                 
-                # Determine RPM limit
+                # Check for concurrency limiting first (takes precedence)
+                if provider_config and 'max_concurrent_calls' in provider_config:
+                    max_concurrent = provider_config['max_concurrent_calls']
+                    if max_concurrent and isinstance(max_concurrent, (int, float)) and max_concurrent > 0:
+                        # Create concurrency limiter
+                        self.limiters[limiter_key] = ConcurrencyLimiter(
+                            max_concurrent=int(max_concurrent),
+                            name=limiter_key
+                        )
+                        
+                        model_info = f"model={provider_config.get('model')}" if 'model' in provider_config else "no model specified"
+                        logger.info(
+                            f"Created concurrency limiter: '{limiter_key}' with max {int(max_concurrent)} concurrent calls "
+                            f"(provider={provider_name}, {model_info})"
+                        )
+                        
+                        return self.limiters[limiter_key]
+                
+                # Fall back to RPM-based rate limiting
                 rpm = self._get_rpm_limit(provider_name, provider_config)
                 
-                # Create new limiter
+                # Create new RPM-based limiter
                 self.limiters[limiter_key] = RateLimiter(
                     requests_per_minute=rpm,
                     provider_name=limiter_key
